@@ -1,5 +1,3 @@
-#![deny(clippy::all, clippy::pedantic, warnings)]
-
 mod cache;
 mod config;
 mod docker;
@@ -231,7 +229,7 @@ fn settings() -> Result<Settings, Failure> {
                 .value_name("REPO")
                 .short("r")
                 .long(DOCKER_REPO_OPTION)
-                .help("Sets the Docker repository"),
+                .help("Sets the Docker repository for remote caching"),
         )
         .arg(
             Arg::with_name(DOCKER_CLI_OPTION)
@@ -249,7 +247,7 @@ fn settings() -> Result<Settings, Failure> {
             Arg::with_name(SHELL_OPTION)
                 .short("s")
                 .long(SHELL_OPTION)
-                .help("Drops you into a shell after the tasks are finished"),
+                .help("Drops you into a containerized shell after the tasks are finished"),
         )
         .arg(
             Arg::with_name(FORCE_OPTION)
@@ -718,7 +716,7 @@ fn entry() -> Result<(), Failure> {
 
     // Return early if needed.
     match result {
-        Ok(_) | Err(Failure::User(_, _)) => {
+        Ok(()) | Err(Failure::User(_, _)) => {
             // Proceed in case the user wants to drop into a shell.
         }
         Err(Failure::Interrupted | Failure::System(_, _)) => {
